package com.kakao.sdk.sample.common

import android.os.Bundle
import android.util.Log
import android.view.*
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.DividerItemDecoration
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.google.android.material.snackbar.BaseTransientBottomBar.*
import com.google.android.material.snackbar.Snackbar
import com.kakao.sdk.auth.*
import com.kakao.sdk.auth.network.TokenBasedApiInterceptor
import com.kakao.sdk.common.model.ClientError
import com.kakao.sdk.common.model.ClientErrorCause
import com.kakao.sdk.common.model.KakaoSdkError
import com.kakao.sdk.common.util.KakaoCustomTabsClient
import com.kakao.sdk.link.LinkClient
import com.kakao.sdk.link.WebSharerClient
import com.kakao.sdk.link.rx
import com.kakao.sdk.navi.NaviClient
import com.kakao.sdk.navi.model.CoordType
import com.kakao.sdk.navi.model.Location
import com.kakao.sdk.navi.model.NaviOptions
import com.kakao.sdk.story.StoryApiClient
import com.kakao.sdk.story.rx
import com.kakao.sdk.talk.TalkApiClient
import com.kakao.sdk.talk.rx
import com.kakao.sdk.template.*
import com.kakao.sdk.user.UserApiClient
import com.kakao.sdk.user.rx
import io.reactivex.Completable
import io.reactivex.Scheduler
import io.reactivex.Single
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.functions.Action
import io.reactivex.functions.Consumer
import io.reactivex.rxkotlin.addTo
import io.reactivex.schedulers.Schedulers
import kotlinx.android.synthetic.main.fragment_open.*

class OpenFragment : Fragment() {

    private lateinit var viewAdapter: RecyclerView.Adapter<*>
    private lateinit var viewManager: RecyclerView.LayoutManager
    private lateinit var disposables: CompositeDisposable

    val onSuccess = Consumer<Any> {
        showSnackbar(it)
    }

    val onError = Consumer<Throwable> {
        showSnackbar(it)
    }

    private fun showSnackbar(any: Any) {
        val snackbar = Snackbar.make(view!!, any.toString(), LENGTH_LONG)
        val layout = snackbar.view as Snackbar.SnackbarLayout
        layout.minimumHeight = 100
        snackbar.show()
    }

    private fun apiItemFrom(
        label: String,
        single: Single<*>,
        scheduler: Scheduler = Schedulers.io()
    ): ApiAdapter.Item.ApiItem =
        ApiAdapter.Item.ApiItem(label) {
            single.subscribeOn(scheduler).subscribe(onSuccess, onError).addTo(disposables)
        }

    private fun apiItemFrom(
        label: String, single: Completable, scheduler: Scheduler = Schedulers.io()
    ): ApiAdapter.Item.ApiItem =
        ApiAdapter.Item.ApiItem(label) {
            single.subscribeOn(scheduler).subscribe(Action {
                showSnackbar("API succeeded.")
            }, onError).addTo(disposables)
        }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        disposables = CompositeDisposable()

        viewManager = LinearLayoutManager(context!!)
        viewAdapter = ApiAdapter(
            listOf(
                ApiAdapter.Item.Header("Kakao Login"),
                apiItemFrom(
                    "Request auth code",
                    AuthCodeClient.rx.authorize(context!!).observeOn(Schedulers.io()).flatMap {
                        AuthApiClient.rx.issueAccessToken(it)
                    }
                ),
                apiItemFrom(
                    "Request auth code with KakaoTalk", AuthCodeClient.rx.authorizeWithTalk(
                        context!!,
                        10012
                    ).observeOn(Schedulers.io()).flatMap {
                        AuthApiClient.rx.issueAccessToken(it)
                    }
                ),
                ApiAdapter.Item.Header("User API"),
                apiItemFrom("me()", UserApiClient.rx.me()),
                apiItemFrom("accessTokenInfo()", UserApiClient.rx.accessTokenInfo()),
                apiItemFrom("shippingAddresses()", UserApiClient.rx.shippingAddresses()),
                apiItemFrom("serviceTerms()", UserApiClient.rx.serviceTerms()),
                apiItemFrom("logout()", UserApiClient.rx.logout()),
                apiItemFrom("unlink()", UserApiClient.rx.unlink()),
                ApiAdapter.Item.Header("KakaoTalk API"),
                apiItemFrom("profile()", TalkApiClient.rx.profile()),
                apiItemFrom(
                    "sendCustomMemo()",
                    TalkApiClient.rx.sendCustomMemo(templateIds["customMemo"] as Long)
                        .retryWhen(
                            TokenBasedApiInterceptor.instance.dynamicAgreement(context!!)
                        )
                ),
                apiItemFrom(
                    "sendDefaultMemo()",
                    TalkApiClient.rx.sendDefaultMemo(defaultFeed)
                        .retryWhen(
                            TokenBasedApiInterceptor.instance.dynamicAgreement(context!!)
                        )
                ),
                apiItemFrom(
                    "sendScrapMemo()",
                    TalkApiClient.rx.sendScrapMemo("https://developers.kakao.com")
                        .retryWhen(
                            TokenBasedApiInterceptor.instance.dynamicAgreement(context!!)
                        )
                ),
                apiItemFrom(
                    "friends()",
                    TalkApiClient.rx.friends()
                        .retryWhen(TokenBasedApiInterceptor.instance.dynamicAgreement(context!!))
//                        .retryWhen {
//                        TokenBasedApiInterceptor.instance.handleInvalidScope(context!!, it)
//                    }
                ),
                apiItemFrom(
                    "sendCustomMessage()",
                    TalkApiClient.rx.friends()
                        .flatMap {
                            FriendsActivity.startFriendsActivityForResult(
                                context,
                                it.elements.map {
                                    PickerItem(
                                        it.uuid,
                                        it.profileNickname,
                                        it.profileThumbnailImage
                                    )
                                })
                        }
                        .observeOn(Schedulers.io())
                        .flatMap {
                            TalkApiClient.rx.sendCustomMessage(
                                it,
                                templateIds["customMessage"] as Long
                            )
                        }.retryWhen(
                            TokenBasedApiInterceptor.instance.dynamicAgreement(context!!)
                        )
                ),
                apiItemFrom(
                    "sendDefaultMessage()",
                    TalkApiClient.rx.friends()
                        .flatMap {
                            FriendsActivity.startFriendsActivityForResult(
                                context,
                                it.elements.map {
                                    PickerItem(
                                        it.uuid,
                                        it.profileNickname,
                                        it.profileThumbnailImage
                                    )
                                })
                        }
                        .observeOn(Schedulers.io())
                        .flatMap { TalkApiClient.rx.sendDefaultMessage(it, defaultFeed) }
                        .retryWhen(TokenBasedApiInterceptor.instance.dynamicAgreement(context!!))
                ),
                apiItemFrom(
                    "sendScrapMessage()",
                    TalkApiClient.rx.friends()
                        .flatMap {
                            FriendsActivity.startFriendsActivityForResult(context, it.elements.map {
                                PickerItem(
                                    it.uuid,
                                    it.profileNickname,
                                    it.profileThumbnailImage
                                )
                            })
                        }
                        .observeOn(Schedulers.io())
                        .flatMap {
                            TalkApiClient.rx.sendScrapMessage(
                                it,
                                "https://developers.kakao.com"
                            )
                        }
                        .retryWhen(
                            TokenBasedApiInterceptor.instance.dynamicAgreement(context!!)
                        )
                ),
                apiItemFrom("plusFriends()", TalkApiClient.rx.plusFriends()),
                ApiAdapter.Item.Header("KakaoStory API"),
                apiItemFrom("isStoryUser()", StoryApiClient.rx.isStoryUser()),
                apiItemFrom("profile()", StoryApiClient.rx.profile()),
                apiItemFrom(
                    "stories()", StoryApiClient.rx.stories().retryWhen(
                        TokenBasedApiInterceptor.instance.dynamicAgreement(context!!)
                    )
                ),
                apiItemFrom("story()", StoryApiClient.rx.stories().flatMap {
                    if (it.isNotEmpty()) {
                        StoryApiClient.rx.story(it[0].id)
                    } else {
                        Single.error<KakaoSdkError>(
                            ClientError(
                                ClientErrorCause.IllegalState,
                                "No stories"
                            )
                        )
                    }
                }.retryWhen(TokenBasedApiInterceptor.instance.dynamicAgreement(context!!))),
                apiItemFrom(
                    "deleteStory()",
                    StoryApiClient.rx.stories().flatMapCompletable {
                        if (it.isNotEmpty()) {
                            StoryApiClient.rx.deleteStory(it[0].id)
                        } else {
                            Completable.error(
                                ClientError(
                                    ClientErrorCause.IllegalState,
                                    "No stories"
                                )
                            )
                        }
                    }),
                apiItemFrom(
                    "linkInfo()",
                    StoryApiClient.rx.linkInfo("https://www.kakaocorp.com")
                ),
                apiItemFrom(
                    "postNote()",
                    StoryApiClient.rx.postNote("Posting note from Kakao SDK Sample.")
                        .retryWhen(
                            TokenBasedApiInterceptor.instance.dynamicAgreement(context!!)
                        )
                ),
                apiItemFrom(
                    "postLink()",
                    StoryApiClient.rx.linkInfo("https://www.kakaocorp.com").flatMap {
                        StoryApiClient.rx.postLink(
                            it,
                            "Posting link from Kakao SDK Sample."
                        )
                    }),
                ApiAdapter.Item.Header("KakaoLink API"),
                apiItemFrom(
                    "isKakaoLinkAvailable()",
                    Single.just(LinkClient.rx.isKakaoLinkAvailable(context = context!!))
                ),
                apiItemFrom(
                    "customTemplate()",
                    LinkClient.rx.customTemplate(
                        context!!,
                        templateIds["customMemo"] as Long
                    ).doOnSuccess {
                        startActivity(it)
                    }),
                apiItemFrom(
                    "customTemplateUri()",
                    Single.just(WebSharerClient.instance.customTemplateUri(templateIds["customMemo"] as Long)).doOnSuccess {
                        Log.e("uri", it.toString())
                        KakaoCustomTabsClient.openWithDefault(
                            context!!,
                            it
                        )
                    }),
                apiItemFrom(
                    "scrapTemplate()",
                    LinkClient.rx.scrapTemplate(
                        context!!,
                        "https://www.kakaocorp.com"
                    ).doOnSuccess {
                        startActivity(it)
                    }),
                apiItemFrom(
                    "scrapTemplateUri()",
                    Single.just(
                        WebSharerClient.instance.scrapTemplateUri(
                            "https://developers.kakao.com"
                        )
                    ).doOnSuccess {
                        Log.e("uri", it.toString())
                        KakaoCustomTabsClient.openWithDefault(
                            context!!,
                            it
                        )
                    }),
                apiItemFrom(
                    "defaultTemplate()", Single.just(defaultFeed).flatMap {
                        LinkClient.rx.defaultTemplate(context!!, it)
                    }.doOnSuccess { startActivity(it) }
                ),
                apiItemFrom("defaultTemplateUri()", Single.just(
                    WebSharerClient.instance.defaultTemplateUri(defaultFeed)
                ).doOnSuccess {
                    KakaoCustomTabsClient.openWithDefault(
                        context!!,
                        it
                    )
                }),
                ApiAdapter.Item.Header("KakaoNavi API"),
                apiItemFrom(
                    "isKakaoNaviInstalled()",
                    Single.fromCallable { NaviClient.instance.isKakaoNaviInstalled(context!!) }
                ),
                apiItemFrom("shareDestinationIntent()", Single.just(
                    NaviClient.instance.shareDestinationIntent(
                        Location("카카오 판교 오피스", 127.10821222694533, 37.40205604363057),
                        NaviOptions(coordType = CoordType.WGS84)
                    )
                ).doOnSuccess {
                    startActivity(it)
                }),
                apiItemFrom(
                    "navigateIntent()", Single.just(
                        NaviClient.instance.navigateIntent(
                            Location("카카오 판교 오피스", 127.10821222694533, 37.40205604363057),
                            NaviOptions(coordType = CoordType.WGS84)
                        )
                    ).doOnSuccess { startActivity(it) }
                ),
                apiItemFrom(
                    "shareDestinationUri()", Single.just(
                        NaviClient.instance.navigateUrl(
                            Location("카카오 판교 오피스", 127.10821222694533, 37.40205604363057),
                            NaviOptions(coordType = CoordType.WGS84)
                        )
                    ).doOnSuccess { KakaoCustomTabsClient.openWithDefault(context!!, it) }
                )
            )
        )
        recyclerView.apply {
            setHasFixedSize(true)
            addItemDecoration(
                DividerItemDecoration(context, DividerItemDecoration.VERTICAL)
            )
            layoutManager = viewManager
            adapter = viewAdapter
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        arguments?.let {
            templateIds = it
        }
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        // Inflate the layout for this fragment
        val view = inflater.inflate(R.layout.fragment_open, container, false)
//        setHasOptionsMenu(true)
        return view
    }

//    override fun onCreateOptionsMenu(menu: Menu, inflater: MenuInflater) {
//        inflater.inflate(R.menu.menu_main, menu)
//        super.onCreateOptionsMenu(menu, inflater)
//
//    }

    override fun onDestroy() {
        disposables.dispose()
        super.onDestroy()
    }

    private lateinit var templateIds: Bundle

    companion object {
        fun newInstance(templateIds: Bundle): OpenFragment =
            OpenFragment().apply {
                arguments = templateIds
            }

        val defaultFeed = FeedTemplate(
            content = Content(
                "딸기 치즈 케익",
                "http://mud-kage.kakao.co.kr/dn/Q2iNx/btqgeRgV54P/VLdBs9cvyn8BJXB3o7N8UK/kakaolink40_original.png",
                Link(
                    "https://developers.kakao.com",
                    "https://developers.kakao.com"
                ),
                description = "#케익 #딸기 #삼평동 #카페 #분위기 #소개팅"
            ),
            social = Social(),
            buttons = listOf(
                Button(
                    "웹으로 보기",
                    Link(
                        "https://developers.kakao.com",
                        "https://developers.kakao.com"
                    )
                ),
                Button(
                    "앱으로 보기",
                    Link(
                        "https://developers.kakao.com",
                        "https://developers.kakao.com",
                        mapOf("key1" to "value1", "key2" to "value2"),
                        mapOf("key1" to "value1", "key2" to "value2")
                    )
                )
            )
        )
    }
}
